@@ -1242,7 +1242,7 @@ gnome_vfs_format_uri_for_display_internal (const char *uri,
  * 	file: uri with fragments should appear as file:uri.
  * 	All other uri appear as expected.
  *
- * Return value: a string which represents @uri and can be displayed.
+ * Return value: a newly allocated string which represents @uri and can be displayed.
  *
  * Since: 2.2
  */
@@ -1624,26 +1624,6 @@ gnome_vfs_uris_match (const char *uri_1, const char *uri_2)
 	return uris_match (uri_1, uri_2, FALSE);
 }
 
-static gboolean
-gnome_vfs_str_has_prefix (const char *haystack, const char *needle)
-{
-        const char *h, *n;
-
-        /* Eat one character at a time. */
-        h = haystack == NULL ? "" : haystack;
-        n = needle == NULL ? "" : needle;
-        do {
-                if (*n == '\0') {
-                        return TRUE;
-                }
-                if (*h == '\0') {
-                        return FALSE;
-                }
-        } while (*h++ == *n++);
-        return FALSE;
-}
-
-
 static gboolean
 gnome_vfs_uri_is_local_scheme (const char *uri)
 {
@@ -1828,8 +1808,9 @@ gnome_vfs_make_uri_canonical (const char *uri)
 	 * typing "foo" into location bar does not crash and returns an error
 	 * rather than displaying the contents of /
 	 */
-	if (gnome_vfs_str_has_prefix (canonical_uri, "file://")
-	    && !gnome_vfs_str_has_prefix (canonical_uri, "file:///")) {
+	if (canonical_uri != NULL
+	    && g_str_has_prefix (canonical_uri, "file://")
+	    && !g_str_has_prefix (canonical_uri, "file:///")) {
 		old_uri = canonical_uri;
 		canonical_uri = g_strconcat ("file:/", old_uri + 5, NULL);
 		g_free (old_uri);
@@ -2002,17 +1983,8 @@ _gnome_vfs_uri_resolve_all_symlinks_uri (GnomeVFSURI *uri,
 				goto out;
 			}
 			escaped_symlink = gnome_vfs_escape_path_string (info->symlink_name);
-			resolved_uri = gnome_vfs_uri_resolve_relative (new_uri,
-								       escaped_symlink);
-			g_free (escaped_symlink);
-			if (resolved_uri == NULL) {
-				gnome_vfs_uri_unref (new_uri);
-				/* FIXME this shouldn't happen. gnome_vfs_uri_resolve_relative
-				 * treats some relative URIs as absolute if they contain special
-				 * characters. We should escape them, or the relative URI recognition
-				 * should be improved */
-				break;
-			}
+			resolved_uri = gnome_vfs_uri_resolve_symbolic_link (new_uri, escaped_symlink);
+			g_assert (resolved_uri != NULL);
 
 			if (*p != 0) {
 				gnome_vfs_uri_unref (new_uri);
@@ -2079,6 +2051,9 @@ gnome_vfs_resolve_symlink (const char *path,
 	int i, j, n;
 	GString *res_path;
 
+	g_assert (path != NULL);
+	g_assert (symlink != NULL);
+
 	p = strrchr (path, '/');
 	
 	if (symlink[0] == '/' || p == NULL) {
